/**
 * @file
 * lazy_image_style behaviors.
 */

(function (Drupal, DrupalSettings, debounce) {

  'use strict';


  const lazyLoadHandler = {

    /**
     * Selector for find lazy object.
     */
    findSelector: 'img[data-src]',

    getConfig: function (key, default_value) {
      return DrupalSettings && DrupalSettings.hasOwnProperty('lazy_image_style') ?
          DrupalSettings.lazy_image_style.hasOwnProperty(key) ?
              DrupalSettings.lazy_image_style[key] : default_value
          : default_value;
    },

    /**
     * Init LazyLoadHandler.
     */
    init: debounce(function () {
      let lazyImages = lazyLoadHandler.getObjects();
      lazyLoadHandler.loadObjects(lazyImages);
    }, 300, true),
    /**
     * Return all imgs suppordet lazy load.
     *
     * @returns {...NodeListOf<Element>[]}
     */
    getObjects: function () {
      return [...document.querySelectorAll(this.findSelector)];
    },
    /**
     * Load objects.
     */
    loadObjects: function (elems) {
      let offset = Number(this.getConfig('offset',100));
      let type = this.getConfig('type','tmp_img');

      elems.forEach(image => {
        image.classList.add('lazy_loaded');
        if ((image.getBoundingClientRect().top <= window.innerHeight + offset
            && image.getBoundingClientRect().bottom >= 0) && getComputedStyle(image).display !== "none") {
          if (type === 'css') {
            // this.restoreData(image);
          }
          image.src = image.dataset.src;
          image.classList.remove('lazy_loaded');
          elems = elems.filter(function (elem) {
            return elem !== image;
          });
          if (elems.length === 0) {
            document.removeEventListener("scroll", lazyLoadHandler.init);
            window.removeEventListener("resize", lazyLoadHandler.init);
            window.removeEventListener("orientationchange", lazyLoadHandler.init);
          }
        }
      })
    },
    restoreData: function (image) {
      image.classList.remove('lazy_image_style');
      image.alt = image.dataset.alt;
      image.title = image.dataset.title;
      image.removeAttribute('style');
    },
  };

  /**
   * Find lazy img and set src from data-src.
   */
  Drupal.behaviors.lazyImage = {

    attach: function (context, settings) {
      lazyLoadHandler.init();
      document.addEventListener("scroll", lazyLoadHandler.init);
      window.addEventListener("resize", lazyLoadHandler.init);
      window.addEventListener("orientationchange", lazyLoadHandler.init);
    },
  };

}(Drupal, drupalSettings, Drupal.debounce));
