<?php

namespace Drupal\lazy_image_style\Form;

use Drupal\Core\Cache\CacheTagsInvalidator;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManager;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\image\Entity\ImageStyle;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure Lazy image settings for this site.
 */
class SettingsForm extends ConfigFormBase {

  /**
   * The entity_type.manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManager
   */
  protected $entityTypeManager;

  /**
   * The cache_tags.invalidator service.
   *
   * @var \Drupal\Core\Cache\CacheTagsInvalidator
   */
  protected $cacheTagsInvalidator;

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'lazy_image_style_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['lazy_image_style.settings'];
  }

  /**
   * SettingsForm constructor.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   * @param \Drupal\Core\Entity\EntityTypeManager $entityTypeManager
   * @param \Drupal\Core\Cache\CacheTagsInvalidator $cacheTagsInvalidator
   */
  public function __construct(ConfigFactoryInterface $config_factory,
                              EntityTypeManager $entityTypeManager,
                              CacheTagsInvalidator $cacheTagsInvalidator) {
    parent::__construct($config_factory);
    $this->entityTypeManager = $entityTypeManager;
    $this->cacheTagsInvalidator = $cacheTagsInvalidator;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('entity_type.manager'),
      $container->get('cache_tags.invalidator')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('lazy_image_style.settings');

    $form['lazy_style'] = [
      '#type' => 'select',
      '#title' => $this->t('Temporary style'),
      '#description' => $this->t('This temporary style used before an image is visible.'),
      '#options' => ['_none' => $this->t('None')] + $this->getStyleList(),
      '#default_value' => $config->get('lazy_style'),
    ];

    $form['offset'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Offset'),
      '#default_value' => $config->get('offset') ?: '100',
      '#description' => $this->t("The offset controls how early you want the elements to be loaded before they're visible. Default is <strong>100</strong>, so <strong>100px</strong> before an element is visible it'll start loading."),
      '#field_suffix' => 'px',
      '#maxlength' => 5,
      '#size' => 10,
    ];

    $form['use_group'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Used styles'),
      '#description' => $this->t('Selected style for use lazy image.'),
    ];

    $form['use_group']['use'] = [
      '#type' => 'checkboxes',
      '#options' => $this->getStyleList(),
      '#default_value' => $config->get('use') ?: [],
    ];
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config('lazy_image_style.settings');
    $old_used = $config->get('use') ?: [];
    $new_used = $form_state->getValue('use', []);
    $config->set('use', $new_used)
      ->set('type', $form_state->getValue('type'))
      ->set('lazy_style', $form_state->getValue('lazy_style'))
      ->set('offset', $form_state->getValue('offset'))
      ->save();

    foreach ($old_used as $style_id => $old_status) {
      if (array_key_exists($style_id, $new_used) && $new_used[$style_id] !== $old_status) {
        $this->cacheTagsInvalidator->invalidateTags(['lazy_image_style:' . $style_id]);
      }
    }

    parent::submitForm($form, $form_state);
  }

  protected function getStyleList() {
    $output = [];
    $img_styles = $this->entityTypeManager->getStorage('image_style')
      ->loadMultiple();
    /**
     * @var string $name
     * @var ImageStyle $img_style
     */
    foreach ($img_styles as $name => $img_style) {
      $output[$name] = $img_style->label();
    }

    return $output;
  }

}
